// ILI9342 SPI Graphics TEST
#include <SPI.h>
#include <Wire.h>
#include "image.h"
//#include "image2.h"
#include "font8x16.h"

#define I2C_ADDR_AXP192         0x34

#define TFT_CLK 18
#define TFT_MISO 38
#define TFT_MOSI 23
#define TFT_DC 15
#define TFT_CS 5

//#define TFT_RST 9   // AXP_IO4
//#define TFT_BL 7    // AXP_DC3
//#define TFT_PWR 7   // AXP_LDO2

uint8_t SPIBuf[320*2] ;   // SPI転送用バッファ
uint8_t fontColorF[2] ;   // TEXT FOR GROND COLOR
uint8_t fontColorB[2] ;   // TEXT BACK GROND COLOR

#define ILI9342_RDMODE 0x0A     ///< Read Display Power Mode
#define ILI9342_RDMADCTL 0x0B   ///< Read Display MADCTL
#define ILI9342_RDPIXFMT 0x0C   ///< Read Display Pixel Format
#define ILI9342_RDIMGFMT 0x0D   ///< Read Display Image Format
#define ILI9342_RDSELFDIAG 0x0F ///< Read Display Self-Diagnostic Result

#define ILI9342_SOFTRESET 0x01              // SOFTWARE RESET
#define ILI9342_SLEEP_OUT 0x11              // SLEEP OUT
#define ILI9342_DISPLAY_OFF 0x28            // DISPLAY OFF
#define ILI9342_DISPLAY_ON 0x29             // DISPLAY ON
#define ILI9342_COLUMN_ADDRESS_SET 0x2a     // COL SET
#define ILI9342_PAGE_ADDRESS_SET 0x2b       // ROW SET
#define ILI9342_MEMORY_WRITE 0x2c           // MEMORY WRITE
#define ILI9342_MEMORY_ACCESS_CONTROL 0x36  // 表示向き

#define ILI9342_NORON 0x13    ///< Normal Display Mode ON
#define ILI9342_PTLON 0x12    ///< Partial Mode ON
#define ILI9342_INVOFF 0x20   ///< Display Inversion OFF
#define ILI9342_INVON 0x21    ///< Display Inversion ON

#define ILI9342_COLMOD 0x3a                 // 表示モード

#define ILI9342_COLOR16 0x55                // 1Dot/16Bitモード
#define ILI9342_COLOR18 0x66                // 1Dot/18Bitモード
# define ILI9342_MAC 0xe8   // 0xe8 / 0x48  // 表示方向　-　横(0xe8)/縦(0x48)
# define ILI9342_WIDTH 320  // 320  / 240
# define ILI9342_HEIGHT 240 // 240  / 320

int tftWidth ;
int tftHeight ;
uint8_t tftReadBuff[16] ;

// ▲▼▲▼▲▼ I2C操作用 ▲▼▲▼▲▼
// 1バイト受信
uint8_t ReadByte(uint8_t i2cAddr , uint8_t addr) {
  Wire1.beginTransmission(i2cAddr);
  Wire1.write(addr);
  Wire1.endTransmission();
  Wire1.requestFrom(i2cAddr, 1);
  while (Wire1.available() < 1) ;
  uint8_t val = Wire1.read();
  return val;
}

// 1バイト送信
void WriteByte(uint8_t i2cAddr , uint8_t addr, uint8_t data) {
  Wire1.beginTransmission(i2cAddr);
  Wire1.write(addr);
  Wire1.write(data);
  Wire1.endTransmission();
}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
// TFTにコマンドを送信
void tftSendCommand(uint8_t command) {
  digitalWrite(TFT_CS, LOW); // SILI9342Cセレクト
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_CS, HIGH);  // SILI9342C解放
}

// TFTにコマンド+1バイトデータを送信
void tftSendCommand1(uint8_t command, uint8_t data1) {
  digitalWrite(TFT_CS, LOW); // SILI9342Cセレクト
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  digitalWrite(TFT_CS, HIGH);  // SILI9342C解放
}

// TFTにコマンド+2バイトデータを送信
void tftSendCommand2(uint8_t command, uint8_t data1, uint8_t data2) {
  digitalWrite(TFT_CS, LOW); // SILI9342Cセレクト
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  digitalWrite(TFT_CS, HIGH);  // SILI9342C解放
}

// TFTにコマンド+3バイトデータを送信
void tftSendCommand3(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3) {
  digitalWrite(TFT_CS, LOW); // SILI9342Cセレクト
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  digitalWrite(TFT_CS, HIGH);  // SILI9342C解放
}

// TFTにコマンド+4バイトデータを送信
void tftSendCommand4(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3, uint8_t data4) {
  digitalWrite(TFT_CS, LOW); // SILI9342Cセレクト
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  SPI.transfer(data4);
  digitalWrite(TFT_CS, HIGH);  // SILI9342C解放
}

// TFTにコマンド+5バイトデータを送信
void tftSendCommand5(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3, uint8_t data4, uint8_t data5) {
  digitalWrite(TFT_CS, LOW); // SILI9342Cセレクト
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  SPI.transfer(data4);
  SPI.transfer(data5);
  digitalWrite(TFT_CS, HIGH);  // SILI9342C解放
}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
uint8_t tftReadCommand(uint8_t command) {
  uint8_t data ;
  digitalWrite(TFT_CS, LOW);  //通信開始
  digitalWrite(TFT_DC, LOW); // Command mode
  // Send Command
  SPI.transfer(command);
  // Read Data
  digitalWrite(TFT_DC, HIGH); // Data mode
  data  = SPI.transfer(0);
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了

  return data ;
}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
void lcd_init() {
  // --- HARDWARE Reset
//  digitalWrite(TFT_RST, LOW);
//  delay(20);
//  digitalWrite(TFT_RST, HIGH);
//  delay(20);    // このdelayは必要

  // --- SOFTWARE Reset
  tftSendCommand(ILI9342_SOFTRESET) ;   // SOFTWARE RESET
  delay(150);                           // このdelayはなくても大丈夫かも

  tftSendCommand(ILI9342_NORON) ; //  Normal Display Mode ON
                          //    このコマンドは、表示を通常モードに戻します。
                          //    通常表示モードのオンは、部分モードのオフを意味します。
                          //    部分モードオンコマンド=(0x12)
  tftSendCommand(ILI9342_INVON) ; //  Display Inversion ON
                          //    このコマンドは、表示反転モードからリカバリーするために使用されます。
                          //    このコマンドは、フレームメモリの内容を変更しません。
                          //    このコマンドは、他の状態を変更しません。
  
  // ----- ILI9431動作モード設定 -----
  screenDir(1) ;

  // ピクセルフォーマットを1Dot/16bitに設定
  tftSendCommand1(ILI9342_COLMOD,ILI9342_COLOR16);

  // PowerOFFにならないモード
  tftSendCommand(ILI9342_SLEEP_OUT);
  delay(60);  // このdelayは必要

  // 画面表示ON
  tftSendCommand(ILI9342_DISPLAY_ON);
}

void screenDir(int dir) {
  //  MY - 0x80 // Row Address Order
  //  MX - 0x40 // Column Address Order
  //  MV - 0x20 // Row / Column Exchange
  //  ML - 0x10 // Vertical Refresh Order
  //  MRGB - 0x08 // RBG - BGR
  //  MH - 0x04 // Horizontal Refresh ORDER
  uint8_t PR = 0 ;

  tftWidth = ILI9342_WIDTH ;
  tftHeight=ILI9342_HEIGHT ;
  switch(dir) {
    case 1:
      tftWidth = ILI9342_WIDTH ;
      tftHeight= ILI9342_HEIGHT ;
      PR = 0x08 ;   // MX | MRGB // DIR - 1 : 縦
      break ;
    case 3:
      tftWidth = ILI9342_WIDTH ;
      tftHeight= ILI9342_HEIGHT ;
      PR = 0xC8;    // MY | MH | MRGB // DIR - 3 : 縦
      break ;
    case 2:
      tftWidth = ILI9342_HEIGHT ;
      tftHeight= ILI9342_WIDTH ;
      PR = 0x68 ;   // MY | MX | MV | MRGB //　dir-2 : 横
      break ;
    case 4:
      tftWidth = ILI9342_HEIGHT ;
      tftHeight= ILI9342_WIDTH ;
      PR = 0xA8 ;   // MV | MRGB //　dir-4 : 横
      break ;
  }
  tftSendCommand1(ILI9342_MEMORY_ACCESS_CONTROL,PR) ;
}

// △▽△▽△▽△▽△▽△▽△▽△▽△▽△▽△▽△▽△▽△▽△▽△▽
void dispStr(char str[] , uint8_t x , uint8_t y) {
  for (int i=0;str[i] != 0 ; i++) {
      dispFont(str[i],x,y) ;
      x += 8 ;
      if (x+8>=tftWidth) {
        x = 0 ;
        y += 16 ;
        if (y + 16 >= tftHeight) {
          y = 0 ; 
        }
      }
  }
}

void dispFont(uint8_t code , uint8_t x, uint8_t y) {
  int pos = (code - 0x20) * 16 ;
  memset(SPIBuf,0,8*16*2) ;
  for (int i=0;i<16;i++) {
    uint8_t data = ssd1306xled_font8x16[pos + i];
    uint8_t bitMast = 0x01 ; 
    for (int j=0;j<8;j++) {
      int fpos = 0 ;
      if (i<8) {
        fpos = (j*8+i)*2 ;
      } else {
        fpos = ((j+8)*8+(i-8))*2 ;
      }
      if ((data & bitMast) == 0) {
        SPIBuf[fpos] = fontColorB[0] ;
        SPIBuf[fpos+1] = fontColorB[1] ;
      } else {
        SPIBuf[fpos] = fontColorF[0] ;
        SPIBuf[fpos+1] = fontColorF[1] ;
      }
      bitMast <<= 1 ;
    }
  }

  uint8_t endX = x + 7 ;
  uint8_t endY = y + 15;
  Serial.print("x-Xe=") ;
  Serial.print(x) ;
  Serial.print("-") ;
  Serial.println(endX) ;
  Serial.print("y-Ye=") ;
  Serial.print(y) ;
  Serial.print("-") ;
  Serial.println(endY) ;

  digitalWrite(TFT_CS, LOW);  //通信開始

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_COLUMN_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(x);
  SPI.transfer16(endX);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_PAGE_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(y);
  SPI.transfer16(endY);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_MEMORY_WRITE);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(SPIBuf, 8*16*2);
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了

}

// BITマップを表示する::基本fillと同じ作り
void dispBitMap(uint16_t startX , uint16_t startY , uint16_t width, uint16_t hight, uint8_t bitMap[]) {
  uint16_t endX = startX + width - 1;
  uint16_t endY = startY + hight - 1;

  digitalWrite(TFT_CS, LOW);  //通信開始

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_COLUMN_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(startX);
  SPI.transfer16(endX);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_PAGE_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(startY);
  SPI.transfer16(endY);

  int pos = 0 ;
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_MEMORY_WRITE);
  digitalWrite(TFT_DC, HIGH); // Command mode
  for (int row = 0 ; row < hight ; row ++) {
    // 表示データ転送
    // Arduinoだとメモリが足りなくて１画面分を一気に転送する事ができない
    // なので、1Lineずつ送信している
    for (int col = 0 ; col < width * 2 ; col ++ ) {
      SPIBuf[col] = bitMap[pos];
      pos ++ ;
    }
    SPI.transfer(SPIBuf, width * 2);
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了
}

void fill(uint16_t xs,uint16_t ys,uint16_t xe,uint16_t ye,uint16_t color) {
  int hight = ye - ys + 1 ;
  int width = xe - xs + 1 ;
  int num = hight * width ;
  
  digitalWrite(TFT_CS, LOW);  //通信開始

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_COLUMN_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(xs);
  SPI.transfer16(xe);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_PAGE_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(ys);
  SPI.transfer16(ye);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_MEMORY_WRITE);
  digitalWrite(TFT_DC, HIGH); // Command mode
  for (int i = 0 ; i < num ; i++) {
    SPI.transfer16(color);
  }

  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了
}

void fillScreen(uint16_t color) {
  digitalWrite(TFT_CS, LOW);  //通信開始

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_COLUMN_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(0);
  SPI.transfer16(tftWidth-1);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_PAGE_ADDRESS_SET);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(0);
  SPI.transfer16(tftHeight-1);

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(ILI9342_MEMORY_WRITE);
  digitalWrite(TFT_DC, HIGH); // Command mode
  for (uint16_t y = 0; y < tftHeight; ++y) {
    for (uint16_t x = 0; x < tftWidth; ++x) {
      SPI.transfer16(color);
    }
  }

  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了
}

void setPartialArea(uint16_t sr,uint16_t er) {
  digitalWrite(TFT_CS, LOW);  //通信開始

  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x12);
  SPI.transfer(0x30);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(sr);
  SPI.transfer16(er);
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了  
}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
void dispTFTStatus( ) {
  Serial.println("--- TFT STATUS ---");
  // read diagnostics (optional but can help debug problems)
  uint8_t x = tftReadCommand(ILI9342_RDMODE);
  Serial.print("Display Power Mode: 0x"); Serial.println(x, HEX);
  if ((x & 0x04) != 0) {
    Serial.println("- Display is On");
  } else {
    Serial.println("- Display is Off");
  }
  if ((x & 0x08) != 0) {
    Serial.println("- Display Normal Mode On");
  } else {
    Serial.println("- Display Normal Mode Off");
  }
  if ((x & 0x10) != 0) {
    Serial.println("- Sleep Out Mode");
  } else {
    Serial.println("- Sleep In Mode");
  }
  if ((x & 0x20) != 0) {
    Serial.println("- Partial Mode On");
  } else {
    Serial.println("- Partial Mode Off");
  }
  if ((x & 0x40) != 0) {
    Serial.println("- Idle Mode On");
  } else {
    Serial.println("- Idle Mode Off");
  }
  if ((x & 0x80) != 0) {
    Serial.println("- Booster On and working OK");
  } else {
    Serial.println("- Booster Off or has a fault");
  }

  x = tftReadCommand(ILI9342_RDMADCTL);
  Serial.print("MADCTL Mode: 0x"); Serial.println(x, HEX);
  if ((x & 0x04) != 0) {
    Serial.println("- LCD Refresh Right to Left");
  } else {
    Serial.println("- LCD Refresh Left to Right");
  }
  if ((x & 0x08) != 0) {
    Serial.println("- BGR (When MADCTL B3=’1’)");
  } else {
    Serial.println("- RGB (When MADCTL B3=’0’)");
  }
  if ((x & 0x10) != 0) {
    Serial.println("- LCD Refresh Bottom to Top (When MADCTL B4=’1’)");
  } else {
    Serial.println("- LCD Refresh Top to Bottom (When MADCTL B4=’0’)");
  }
  if ((x & 0x20) != 0) {
    Serial.println("- Reverse Mode (When MADCTL B5=’1’)");
  } else {
    Serial.println("- Normal Mode (When MADCTL B5=’0’)");
  }
  if ((x & 0x40) != 0) {
    Serial.println("- Right to Left (When MADCTL B6=’1’)");
  } else {
    Serial.println("- Left to Right (When MADCTL B6=’0’)");
  }
  if ((x & 0x80) != 0) {
    Serial.println("- Bottom to Top (When MADCTL B7=’1’)");
  } else {
    Serial.println("- Top to Bottom (When MADCTL B7=’0’)");
  }

  x = tftReadCommand(ILI9342_RDPIXFMT);
  Serial.print("Pixel Format: 0x"); Serial.println(x, HEX);
  if ((x & 0x10) != 0) {
    Serial.println("- RGB 16bits/pixel");
  } else {
    Serial.println("- RGB 18bits/pixel");
  }
  if ((x & 0x01) != 0) {
    Serial.println("- MCU 16bits/pixel");
  } else {
    Serial.println("- MCU 18bits/pixel");
  }

  x = tftReadCommand(ILI9342_RDIMGFMT);
  Serial.print("Image Format: 0x"); Serial.println(x, HEX);

  x = tftReadCommand(ILI9342_RDSELFDIAG);
  Serial.print("Self Diagnostic: 0x"); Serial.println(x, HEX); 
  Serial.println() ;
}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
// SPISettings settings = SPISettings(8000000, MSBFIRST, SPI_MODE0);

void setup() {
  Serial.begin(9600);
  delay(2000) ;
  Serial.println(F("ILI9342 Init Test"));
  Serial.println() ;
  Serial.println() ;

  // --- Power Module : 
  Wire1.begin(21, 22);
  uint8_t val = ReadByte(I2C_ADDR_AXP192,0x12);
  Serial.print("AXP192 Register 0x12:") ;
  Serial.println(val,HEX) ;
  val = ReadByte(I2C_ADDR_AXP192,0x27);
  Serial.print("AXP192 Register 0x27:") ;
  Serial.println(val,HEX) ;
  val = ReadByte(I2C_ADDR_AXP192,0x28);
  Serial.print("AXP192 Register 0x28:") ;
  Serial.println(val,HEX) ;

  // TFT_RST - AXP_IO4
  // TFT_BL  - AXP_DC3   - 0x08
  // TFT_PWR - AXP_LDO2  - 0x04
  val = (val | 0xB7) &0xF7;  
  WriteByte(I2C_ADDR_AXP192 , 0x12, val);
  // TFT & BACK LIGHT POWER ON
  WriteByte(I2C_ADDR_AXP192 , 0x27, 0xCC); // BakcLight - D4
  WriteByte(I2C_ADDR_AXP192 , 0x28, 0xF2); // TFT-POWER - F2

  // ---- SPI
  SPI.begin();  //SPIを初期化、SCK、MOSI、SSの各ピンの動作は出力、SCK、MOSIはLOW、SSはHIGH
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(SPI_MODE0);

  // -----  ILI9342C INITIAL -----
  pinMode(TFT_CS, OUTPUT);
  pinMode(TFT_DC, OUTPUT);
//  pinMode(TFT_RST, OUTPUT);

  lcd_init( ) ;

  dispTFTStatus( ) ;

}

// ▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼▲▼
void loop() {
  tftSendCommand(ILI9342_NORON) ; //  Normal Display Mode ON
  screenDir(1) ;

  fillScreen(0xF800) ;  // B11111 000 000 00000 : RED
  delay(500) ;
  fillScreen(0x07E0) ;  // B00000 111 111 00000 : GREEN
  delay(500) ;
  fillScreen(0x001F) ;  // B00000 000 000 11111 : BLUE
  delay(500) ;

  // ----- TEXT -----
  fillScreen(0xFFFF) ;  // B11111 111 111 11111 :  WHITE
  fontColorB[0] = 0xFF ; fontColorB[1] = 0xFF ; // TEXT BACK GROND COLOR
  fontColorF[0] = 0xFF ; fontColorF[1] = 0xCF ; // TEXT FOR GROND COLOR - WHITE
  dispStr("ABCDEFGHIJ",0,0) ;
  fontColorF[0] = 0x07 ; fontColorF[1] = 0xE0 ; // TEXT FOR GROND COLOR - GREEN
  dispStr("0123456789",0,16) ;
  fontColorF[0] = 0x00 ; fontColorF[1] = 0x1F ; // TEXT FOR GROND COLOR - BLUE
  dispStr("ABCDEFGHIJ",0,32) ;
  fontColorF[0] = 0xF8 ; fontColorF[1] = 0x00 ; // TEXT FOR GROND COLOR - RED
  dispStr("0123456789",0,48) ;
  fontColorF[0] = 0xFE ; fontColorF[1] = 0xE0 ; // TEXT FOR GROND COLOR - YELLOW
  dispStr("ABCDEFGHIJ",0,64) ;
  fontColorF[0] = 0x07 ; fontColorF[1] = 0xFF ; // TEXT FOR GROND COLOR - CIAN
  dispStr("0123456789",0,80) ;
  delay(1000) ;

  // --- SCROLL
  for (int i=0, sc=16;i<10;i++,sc+=16) {
    digitalWrite(TFT_CS, LOW);  //通信開始
    digitalWrite(TFT_DC, LOW); // Command mode
    SPI.transfer(0x37);
    digitalWrite(TFT_DC, HIGH); // Command mode
    SPI.transfer16(sc);
    
    digitalWrite(TFT_DC, LOW); // Command mode
    digitalWrite(TFT_CS, HIGH); //通信終了  
    delay(1000) ;
  }
  digitalWrite(TFT_CS, LOW);  //通信開始
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x37);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer16(0);
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH); //通信終了  

  delay(2000) ;

//  setPartialArea(50,100) ;
//  fillScreen(0xFFE0) ;  // B11111 111 111 00000 : YELLOW
//  delay(2000) ;
//
//  setPartialArea(150,200) ;
//  fillScreen(0x07FF) ;  // B00000 111 111 11111 : CIAN
//  delay(2000) ;
//
//  setPartialArea(100,150) ;
//  fillScreen(0xF81F) ;  // B11111 000 000 11111 : MAGENTA
//  delay(2000) ;
//
//  tftSendCommand(ILI9342_NORON) ; //  Normal Display Mode ON
//  fillScreen(0xFFFF) ;  // B11111 111 111 11111 :  WHITE
//  delay(2000) ;

  // --- BITMAP ---
  fontColorB[0] = 0x07 ; fontColorB[1] = 0xE0 ; // TEXT BACK GROND COLOR
  fontColorF[0] = 0x00 ; fontColorF[1] = 0x00 ; // TEXT FOR GROND COLOR - WHITE
  fillScreen(0x07E0) ; 
  screenDir(1) ;
  dispStr("DIR-1",0,0) ;
  dispBitMap(0,   20, 240, 179, bitmapData) ; delay(2000) ;

  fontColorB[0] = 0xFF ; fontColorB[1] = 0xFF ; // TEXT BACK GROND COLOR
  fontColorF[0] = 0x00 ; fontColorF[1] = 0x00 ; // TEXT FOR GROND COLOR - WHITE
  fillScreen(0xFFFF) ; 
  screenDir(2) ;
  dispStr("DIR-2",0,0) ;
  dispBitMap(0,   20, 240, 179, bitmapData) ; delay(2000) ;

  fontColorB[0] = 0x07 ; fontColorB[1] = 0xE0 ; // TEXT BACK GROND COLOR
  fontColorF[0] = 0x00 ; fontColorF[1] = 0x00 ; // TEXT FOR GROND COLOR - WHITE
  fillScreen(0x07E0) ; 
  screenDir(3) ;
  dispStr("DIR-3",0,0) ;
  dispBitMap(0,   20, 240, 179, bitmapData) ; delay(2000) ;

  fontColorB[0] = 0xFF ; fontColorB[1] = 0xFF ; // TEXT BACK GROND COLOR
  fontColorF[0] = 0x00 ; fontColorF[1] = 0x00 ; // TEXT FOR GROND COLOR - WHITE
  fillScreen(0xFFFF) ; 
  screenDir(4) ;
  dispStr("DIR-4",0,0) ;
  dispBitMap(0,   20, 240, 179, bitmapData) ; delay(2000) ;

  screenDir(2) ;

  // --
  tftSendCommand1(ILI9342_MEMORY_ACCESS_CONTROL,0xe8) ;
  fillScreen(0x0000) ;  // B00000 000 000 00000 :  BLACK
  for (int y=0;y<ILI9342_HEIGHT;y+=18) {
    fill(0,y,ILI9342_WIDTH-1,y+8,0x07E0) ; // B00000 111 111 00000 : GREEN
  }
  for (int x=0;x<ILI9342_WIDTH;x+=16) {
    fill(x,0,x+8,ILI9342_HEIGHT-1,0xFFE0) ; // B00000 111 111 00000 : GREEN
  }
  delay(2000) ;

}
